import time
import board
import digitalio

# DS1302 寄存器地址
SECONDS_REG = 0x80
MINUTES_REG = 0x82
HOURS_REG = 0x84
DATE_REG = 0x86
MONTH_REG = 0x88
DAY_REG = 0x8A
YEAR_REG = 0x8C
CONTROL_REG = 0x8E
CLOCK_BURST = 0xBE


class DS1302:
    def __init__(self, clk_pin, data_pin, rst_pin):
        self.clk = digitalio.DigitalInOut(clk_pin)
        self.data = digitalio.DigitalInOut(data_pin)
        self.rst = digitalio.DigitalInOut(rst_pin)

        self.clk.direction = digitalio.Direction.OUTPUT
        self.rst.direction = digitalio.Direction.OUTPUT
        self.data.direction = digitalio.Direction.OUTPUT

    def _write_byte(self, data):
        for i in range(8):
            self.data.value = (data >> i) & 1
            self.clk.value = True
            time.sleep(0.00001)
            self.clk.value = False
            time.sleep(0.00001)

    def _read_byte(self):
        value = 0
        self.data.direction = digitalio.Direction.INPUT
        for i in range(8):
            if self.data.value:
                value |= (1 << i)
            self.clk.value = True
            time.sleep(0.00001)
            self.clk.value = False
            time.sleep(0.00001)
        self.data.direction = digitalio.Direction.OUTPUT
        return value

    def _start(self):
        self.rst.value = True
        time.sleep(0.00001)

    def _stop(self):
        self.rst.value = False
        time.sleep(0.00001)

    def write_register(self, reg, data):
        self._start()
        self._write_byte(reg)
        self._write_byte(data)
        self._stop()

    def read_register(self, reg):
        self._start()
        self._write_byte(reg | 0x01)
        value = self._read_byte()
        self._stop()
        return value

    def get_time(self):
        seconds = self._bcd_to_dec(self.read_register(SECONDS_REG) & 0x7F)
        minutes = self._bcd_to_dec(self.read_register(MINUTES_REG) & 0x7F)
        hours = self._bcd_to_dec(self.read_register(HOURS_REG) & 0x3F)
        date = self._bcd_to_dec(self.read_register(DATE_REG) & 0x3F)
        month = self._bcd_to_dec(self.read_register(MONTH_REG) & 0x1F)
        day = self._bcd_to_dec(self.read_register(DAY_REG) & 0x07)
        year = self._bcd_to_dec(self.read_register(YEAR_REG))
        return {
            'year': 2000 + year,
            'month': month,
            'date': date,
            'day': day,
            'hours': hours,
            'minutes': minutes,
            'seconds': seconds
        }

    def set_time(self, year, month, date, day, hours, minutes, seconds):
        self.write_register(SECONDS_REG, self._dec_to_bcd(seconds))
        self.write_register(MINUTES_REG, self._dec_to_bcd(minutes))
        self.write_register(HOURS_REG, self._dec_to_bcd(hours))
        self.write_register(DATE_REG, self._dec_to_bcd(date))
        self.write_register(MONTH_REG, self._dec_to_bcd(month))
        self.write_register(DAY_REG, self._dec_to_bcd(day))
        self.write_register(YEAR_REG, self._dec_to_bcd(year - 2000))

    def _dec_to_bcd(self, val):
        return ((val // 10) << 4) + (val % 10)

    def _bcd_to_dec(self, val):
        return ((val >> 4) * 10) + (val & 0x0F)


# 初始化 DS1302
clk_pin = board.GP8   # The clock pin is connected to GP8
data_pin = board.GP7  # Data pin connected to GP7
rst_pin = board.GP6   # Reset pin connected to GP6

rtc = DS1302(clk_pin, data_pin, rst_pin)

#Set Time
#rtc.set_time(2024, 11, 29, 5, 10, 30, 0)
# 获取当前时间
while True:
    current_time = rtc.get_time()
    print("Time: {}-{:02d}-{:02d} {:02d}:{:02d}:{:02d}".
          format(current_time['year'],
                 current_time['month'],
                 current_time['date'],
                 current_time['hours'],
                 current_time['minutes'],
                 current_time['seconds']))
    time.sleep(1)
